/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.context;

import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLDataFactory;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyManager;

/**
 *
 * @author seylan
 */
public class Context {

    private final Reasoning reasoning;
    private final OWLOntologyManager manager;
    private final OriginalOntology originalOnt;
    private final OmegaContext omegaContext;

    private final OWLClassExpression topConcept;

    public Context(OWLOntology ont, OWLOntologyManager manager) {
        this.manager = manager;
        originalOnt = new OriginalOntology(ont, manager);
        reasoning = new Reasoning(this, originalOnt.getGCIs(), originalOnt.getRoleInclusions());
        omegaContext = new OmegaContext(this);
        topConcept = OWLManager.getOWLDataFactory().getOWLThing();
    }

    public OWLClassExpression topConcept() {
        return topConcept;
    }

    public OmegaContext getOmegaContext() {
        return omegaContext;
    }

    public OriginalOntology getOriginalOnt() {
        return originalOnt;
    }

    public Reasoning getReasoning() {
        return reasoning;
    }

    public OWLDataFactory getFactory() {
        return manager.getOWLDataFactory();
    }

    public OWLOntologyManager getManager() {
        return manager;
    }
}
