/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.gammaconstruction;

import java.util.Iterator;
import java.util.Objects;
import java.util.SortedSet;
import org.semanticweb.owlapi.model.OWLClassExpression;

/**
 *
 * @author hansen
 */
public final class GammaTuple {

    private final OWLClassExpression concept;
    private final SortedSet<OWLClassExpression> setOfConcepts;

    public GammaTuple(OWLClassExpression concept,
            SortedSet<OWLClassExpression> setOfConcepts) {
        this.concept = concept;
        this.setOfConcepts = setOfConcepts;
    }

    public OWLClassExpression getConcept() {
        return this.concept;
    }

    public SortedSet<OWLClassExpression> getSetOfConcepts() {
        return this.setOfConcepts;
    }

    @Override
    public String toString() {
        String result = "(" + this.concept.toString() + ", {";
        for (Iterator<OWLClassExpression> conceptIter = this.setOfConcepts.iterator();
                conceptIter.hasNext();) {
            result += conceptIter.next().toString();
            if (conceptIter.hasNext()) {
                result += ", ";
            }
        }
        result += "})";
        return result;
    }

    private volatile int hash;

    @Override
    public int hashCode() {
        if (hash == 0) {
            hash = 3;
            hash = 59 * hash + Objects.hashCode(this.concept);
            hash = 59 * hash + Objects.hashCode(this.setOfConcepts);
        }
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final GammaTuple other = (GammaTuple) obj;
        if (!Objects.equals(this.concept, other.concept)) {
            return false;
        }
        if (!Objects.equals(this.setOfConcepts, other.setOfConcepts)) {
            return false;
        }
        return true;
    }

}
