/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.runnables;

import de.unibremen.informatik.tdki.grind.context.Context;
import de.unibremen.informatik.tdki.grind.gammaconstruction.Gamma;
import de.unibremen.informatik.tdki.grind.gammaconstruction.GammaGraphCreator;
import de.unibremen.informatik.tdki.grind.gammaconstruction.GammaSaturator;
import de.unibremen.informatik.tdki.grind.gammaconstruction.GammaTuple;
import de.unibremen.informatik.tdki.grind.statistics.DetailsLogger;
import java.awt.*;
import java.awt.geom.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Set;

import javax.swing.*;

import org.jgraph.*;
import org.jgraph.graph.*;

import org.jgrapht.*;
import org.jgrapht.ext.*;
import org.jgrapht.graph.*;

// resolve ambiguity
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyCreationException;
import org.semanticweb.owlapi.model.OWLOntologyManager;

public class GammaGraphVisualiser
        extends JApplet {

    private static final long serialVersionUID = 3256444702936019250L;
    private static final Color DEFAULT_BG_COLOR = Color.decode("#FAFBFF");
    private static final Dimension DEFAULT_SIZE = new Dimension(1024, 768);
    //
    private JGraphModelAdapter<GammaTuple, GammaGraphCreator.RoleEdge> jgAdapter;

    /**
     * An alternative starting point for this demo, to also allow running this
     * applet as an application.
     *
     * @param args ignored.
     */
    public static void main(String[] args) {
        GammaGraphVisualiser applet = new GammaGraphVisualiser();
        applet.init();

        JFrame frame = new JFrame();
        frame.getContentPane().add(applet);
        frame.setTitle("Showing representation of Gamma graph");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.pack();
        frame.setVisible(true);
    }

    /**
     * {@inheritDoc}
     */
    public void init() {
        DirectedPseudograph g;

        try {

            String ontSource = "./ontologies/petersGammaTest.owl";
            g = runGammaAndOmega(ontSource, "A0");
//        runGammaAndOmega(ontSource, "A0");
//        String ontSource = "./ontologies/petersNonFOrewr.owl";
//        runGammaAndOmega(ontSource, "A0");

//        String ontSource = "./ontologies/petersEquivalenceTest.owl";
//        String ontSource = "./ontologies/AAO.owl";
//        String ontSource = "./ontologies/EDAM.obo";
//        String ontSource = "./ontologies/fly_anatomy.owl";
//        String ontSource = "./ontologies/FBbi.owl";
//        String ontSource = "./ontologies/framenet.owl";
//        String ontSource = "./ontologies/MOHSE.owl";
//        String ontSource = "./ontologies/not-galen.owl";
//        String ontSource = "./ontologies/gamma-cycles/anatomy-random-sample-1.owl";

        } catch (OWLOntologyCreationException | FileNotFoundException ex) {
            throw new RuntimeException(ex);
        }

        ListenableGraph g2 = new DefaultListenableGraph(g);

        // create a visualization using JGraph, via an adapter
        jgAdapter = new JGraphModelAdapter<>(g2);

        JGraph jgraph = new JGraph(jgAdapter);

        int numberOfVertices = g2.vertexSet().size();

        // Do some math: Arrange the vertices on a circle, center is 
        // (DEFAULT_SIZE.width() / 2), (DEFAULT_SIZE.height() / 2),
        // radius is (DEFAULT_SIZE.height / 2.5),
        // start with first vertex at 0 degrees and go on until the 
        // last one at ((360/g2.vertexSet().size()) * (g2.vertexSet() - 1)).
        int i = 0;
        for (GammaTuple tuple : (Set<GammaTuple>) g2.vertexSet()) {

            double degree = ((2 * Math.PI) / ((double) g2.vertexSet().size())) * i;
            positionVertexAt(tuple,
                    (int) ((DEFAULT_SIZE.width / 2) + (Math.cos(degree) * (DEFAULT_SIZE.height / 2.5))),
                    (int) ((DEFAULT_SIZE.height / 2) + (Math.sin(degree) * (DEFAULT_SIZE.height / 2.5))));
            i++;
        }

        adjustDisplaySettings(jgraph);
        getContentPane().add(jgraph);
        resize(DEFAULT_SIZE);


        // that's all there is to it!...
    }

    private void adjustDisplaySettings(JGraph jg) {
        jg.setPreferredSize(DEFAULT_SIZE);

        Color c = DEFAULT_BG_COLOR;
        String colorStr = null;

        try {
            colorStr = getParameter("bgcolor");
        } catch (Exception e) {
        }

        if (colorStr != null) {
            c = Color.decode(colorStr);
        }

        jg.setBackground(c);
    }

    @SuppressWarnings("unchecked") // FIXME hb 28-nov-05: See FIXME below
    private void positionVertexAt(Object vertex, int x, int y) {
        DefaultGraphCell cell = jgAdapter.getVertexCell(vertex);
        AttributeMap attr = cell.getAttributes();
        Rectangle2D bounds = GraphConstants.getBounds(attr);

        Rectangle2D newBounds =
                new Rectangle2D.Double(
                x,
                y,
                bounds.getWidth(),
                bounds.getHeight());

        GraphConstants.setBounds(attr, newBounds);

        // TODO: Clean up generics once JGraph goes generic
        AttributeMap cellAttr = new AttributeMap();
        cellAttr.put(cell, attr);
        jgAdapter.edit(cellAttr, null, null, null);
    }

    /**
     * a listenable directed multigraph that allows loops and parallel edges.
     */
    private static class ListenableDirectedMultigraph<V, E>
            extends DefaultListenableGraph<V, E>
            implements DirectedGraph<V, E> {

        private static final long serialVersionUID = 1L;

        ListenableDirectedMultigraph(Class<E> edgeClass) {
            super(new DirectedMultigraph<V, E>(edgeClass));
        }
    }

    private static DirectedPseudograph runGammaAndOmega(String ontSource, String startConceptString) throws OWLOntologyCreationException, FileNotFoundException {

        long t0, t;
        t0 = System.currentTimeMillis(); // ******** timing point ********

        System.out.println("\n\nOpening ontology " + ontSource + "...");
        OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
        File file = new File(ontSource);
        OWLOntology ont = manager.loadOntologyFromOntologyDocument(file);
        Context context = new Context(ont, manager);
        manager.removeOntology(ont);
        System.out.println("  Normalising the ontology: " + (System.currentTimeMillis() - t0) + " ms.");
//        System.out.println("\n  Size of sub(T):           " + context.getFirstNF().getSubconcepts().size());

        // Gamma part from here -----------------------------------------------
        Set<OWLClass> conceptNames = context.getOriginalOnt().getConceptNames();
        int noOfCNs = conceptNames.size();

        PrintWriter output = new PrintWriter(new File("logs/" + file.getName()));
//        PrintWriter output = new PrintWriter(System.out);
        DetailsLogger.getInstance().setWriter(output);
        // Iterate over all classes in ont


        OWLClass startConcept = null;
        boolean found = false;
        for (OWLClass concept : conceptNames) {
            if (concept.getIRI().getFragment().equals(startConceptString)) {
                startConcept = concept;
                found = true;
                break;
            }
        }
        if (!found) {
            throw new RuntimeException("startConceptString not found: " + startConceptString);
        }

        System.out.println("\nWorking on class " + startConcept.getIRI().getFragment());
        GammaSaturator gammaSat = new GammaSaturator(startConcept, context);

        t = System.currentTimeMillis(); // ******** timing point ********
        gammaSat.saturate(15000);
        Gamma gamma = gammaSat.getGamma();
        //System.out.println(  "  Gamma saturation:         " + (System.currentTimeMillis() - t) + " ms.");

        t = System.currentTimeMillis(); // ******** timing point ********
        GammaGraphCreator ggc = new GammaGraphCreator(gamma, startConcept);
        DirectedPseudograph graph = ggc.getGraph();
        System.out.println("\nGamma graph:");
        System.out.println("  Number of vertices:      " + graph.vertexSet().size());
        System.out.println("  Number of edges:         " + graph.edgeSet().size());
        System.out.println("  Gamma graph calculation: " + (System.currentTimeMillis() - t) + " ms.");

        t = System.currentTimeMillis(); // ******** timing point ********
        System.out.println("  Graph is cyclic:         " + ggc.graphIsCyclic());
        System.out.println("  Cyclicity check:         " + (System.currentTimeMillis() - t) + " ms.");

        // Omega part from here -----------------------------------------------

//        System.out.println("\nOmega:");
//        t = System.currentTimeMillis(); // ******** timing point ********
//        OmegaContext omegaContext = new OmegaContext(context);
//        System.out.println("  Omega context:  " + (System.currentTimeMillis() - t) + " ms.");

//            Omega omega0 = new Omega(gammaSat.getGamma(), context);
//            OmegaSaturator omegaSat = new OmegaSaturator(omega0, context);
//            omegaSat.saturate();

        output.close();
        System.out.println("\nTotal time: " + (System.currentTimeMillis() - t0) + " ms.\n");

        return graph;
    }
}
