/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.statistics;

import org.semanticweb.owlapi.model.OWLClass;

/**
 *
 * @author seylan
 */
public class DetailsLogEntry {

    private OWLClass conceptName;
    private boolean gammaTimeout;
    private boolean omegaTimeout;
    // From Gamma:
    private int noOfGammaTuples;
    private int noOfDifferentGammaCConcepts;
    private int[] cardinalitiesOfGammaSSets;
    private int maxGammaSSetCardinality;
    private boolean cyclicGamma;
    private int noOfGammaTuplesOnCycle;
    // From Omega:
    private int noOfOmegaTuples;
    private boolean isFOrewritable;

    // New fields (2015):
    private long timeSpent;

    private final static String SEPARATOR = ",";

    public DetailsLogEntry() {
        resetGammaSSetCardinalities();
    }

    public OWLClass getConceptName() {
        return conceptName;
    }

    public void setConceptName(OWLClass conceptName) {
        this.conceptName = conceptName;
    }

    public boolean isGammaTimeout() {
        return gammaTimeout;
    }

    public void setGammaTimeout(boolean gammaTimeout) {
        this.gammaTimeout = gammaTimeout;
    }

    public boolean isOmegaTimeout() {
        return omegaTimeout;
    }

    public void setOmegaTimeout(boolean omegaTimeout) {
        this.omegaTimeout = omegaTimeout;
    }

    public int getNoOfGammaTuples() {
        return noOfGammaTuples;
    }

    public void setNoOfGammaTuples(int noOfGammaTuples) {
        this.noOfGammaTuples = noOfGammaTuples;
    }

    public int getNoOfDifferentGammaCConcepts() {
        return noOfDifferentGammaCConcepts;
    }

    public void setNoOfDifferentGammaCConcepts(int noOfDifferentGammaCConcepts) {
        this.noOfDifferentGammaCConcepts = noOfDifferentGammaCConcepts;
    }

    public boolean isCyclicGamma() {
        return cyclicGamma;
    }

    public void setCyclicGamma(boolean cyclicGamma) {
        this.cyclicGamma = cyclicGamma;
    }

    public int getNoOfGammaTuplesOnCycle() {
        return noOfGammaTuplesOnCycle;
    }

    public void setNoOfGammaTuplesOnCycle(int noOfGammaTuplesOnCycle) {
        this.noOfGammaTuplesOnCycle = noOfGammaTuplesOnCycle;
    }

    public int getNoOfOmegaTuples() {
        return noOfOmegaTuples;
    }

    public void setNoOfOmegaTuples(int noOfOmegaTuples) {
        this.noOfOmegaTuples = noOfOmegaTuples;
    }

    public boolean isIsFOrewritable() {
        return isFOrewritable;
    }

    public void setIsFOrewritable(boolean isFOrewritable) {
        this.isFOrewritable = isFOrewritable;
    }

    public void resetGammaSSetCardinalities() {
        maxGammaSSetCardinality = 0;
        cardinalitiesOfGammaSSets = new int[10000];
    }

    public void increaseGammaSSetWithCardinality(int cardinality) {
        if (cardinality > maxGammaSSetCardinality) {
            maxGammaSSetCardinality = cardinality;
        }
        cardinalitiesOfGammaSSets[cardinality]++;
    }

    public int getMaxGammaSSetCardinality() {
        return maxGammaSSetCardinality;
    }

    public int getNoOfGammaSSetsWithCardinality(int cardinality) {
        return cardinalitiesOfGammaSSets[cardinality];
    }

    public int getSumGammaSSetCardinality() {
        int result = 0;
        for (int i = 0; i <= maxGammaSSetCardinality; i++) {
            result += i * cardinalitiesOfGammaSSets[i];
        }
        return result;
    }

    public float getAverageGammaSSetCardinality() {
        int occurrences = 0;
        for (int i = 0; i <= maxGammaSSetCardinality; i++) {
            occurrences += cardinalitiesOfGammaSSets[i];
        }
        return ((float) getSumGammaSSetCardinality()) / occurrences;
    }

    public long getTimeSpent() {
        return timeSpent;
    }

    public void setTimeSpent(long timeSpent) {
        this.timeSpent = timeSpent;
    }
    
    

//    OWLClass conceptName;
//    // From Gamma:
//    gammaTimeout/omegaTimeout/success;
//    int noOfGammaTuples;
//    int noOfDifferentGammaLeftHandSides;
//    int[] cardinalitiesOfGammaRightHandSides;
//    int maxGammaRhsCardinality;
//    boolean cyclicGamma;
//    int noOfGammaTuplesOnCycle;
//    // From Omega:
//    int noOfOmegaTuples;
//    boolean isFOrewritable;
    @Override
    public String toString() {
        String result = conceptName.getIRI().getFragment() + SEPARATOR;

        if (gammaTimeout) {
            result += "gammaTimeout" + SEPARATOR;
        } else if (omegaTimeout) {
            result += "omegaTimeout" + SEPARATOR;
        } else {
            result += "success" + SEPARATOR;
        }

        result += noOfGammaTuples + SEPARATOR;
        result += noOfDifferentGammaCConcepts + SEPARATOR;
        result += maxGammaSSetCardinality + SEPARATOR;
        result += getSumGammaSSetCardinality() + SEPARATOR;
        result += getAverageGammaSSetCardinality() + SEPARATOR;

        if (cyclicGamma) {
            result += "cyclicYes" + SEPARATOR;
        } else {
            result += "cyclicNo" + SEPARATOR;
        }
        result += noOfGammaTuplesOnCycle + SEPARATOR;

        result += noOfOmegaTuples + SEPARATOR;

        if (gammaTimeout || omegaTimeout) {
            result += "dunno?"+ SEPARATOR;
        } else if (isFOrewritable) {
            result += "FOyes"+ SEPARATOR;
        } else {
            result += "FOno"+ SEPARATOR;
        }
        
        result += timeSpent;

        return result;
    }
}
