/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.statistics;

import de.unibremen.informatik.tdki.grind.context.Context;
import de.unibremen.informatik.tdki.grind.utilities.StringTools;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyCreationException;
import org.semanticweb.owlapi.model.OWLOntologyManager;

/**
 *
 * @author hansen
 */
public class OntologyComparator {

    public static void main(String[] args) throws OWLOntologyCreationException, FileNotFoundException, Exception {

        // Run selected ontologies:
        String[] ontSources = new String[]{ //            "./ontologies/petersCancelling.owl",
            //            "./ontologies/petersGammaTest.owl",
            //            "./ontologies/petersEquivalenceTest.owl",
            //            "./ontologies/no-cycles/AAO.owl", //    no ex lhs
            //            "./ontologies/no-cycles/EDAM.obo", //    no ex lhs
            //            "./ontologies/no-cycles/fly_anatomy.owl", //    no ex lhs
            //            "./ontologies/framenet.owl",
            //            "./ontologies/cyclic-ontologies/NBO.owl",
            "./ontologies/cyclic-ontologies/ENVO-resaved-protege-402.owl", //                    "./ontologies/cyclic-ontologies/FBbi.owl",
        //            "./ontologies/cyclic-ontologies/MOHSE.owl",
        //                    "./ontologies/cyclic-ontologies/not-galen.owl",
        //                    "./ontologies/cyclic-ontologies/SO.owl",

        //            "../REQUIEM/requiem-tests/cyclic-ontologies/SO.owl",
        //            "../REQUIEM/requiem-tests/cyclic-ontologies/SO-from-obo-402.owl",

        //            "../REQUIEM/requiem-tests/cyclic-ontologies/ENVO.owl",
        //            "../REQUIEM/requiem-tests/cyclic-ontologies/.owl",
        //
        };

        List<Set<String>> listOfConceptnameSets = new ArrayList<>();
        for (String s : ontSources) {

            OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
            File file = new File(s);
            OWLOntology ont = null;
            try {
                ont = manager.loadOntologyFromOntologyDocument(file);
            } catch (Exception e) {
                System.err.println("Error: Could not load ontology: " + e);
                System.exit(-1);
            }
            Context context = new Context(ont, manager);
            manager.removeOntology(ont);
            Set<OWLClass> conceptNames;
            conceptNames = context.getOriginalOnt().getConceptNames();
            Set<String> nameSet = new HashSet<>();
            listOfConceptnameSets.add(nameSet);
            for (OWLClass concept : conceptNames) {

                String name = StringTools.getFragment(concept.getIRI());
                nameSet.add(name);
                System.out.print(name + ", ");
            }
        }
        for (String name : listOfConceptnameSets.get(0)) {
            if (!listOfConceptnameSets.get(1).toString().contains(name)) {
                System.out.println("Concept " + name + " not contained in SO-from-obo.402.owl!");
            }
        }
        System.out.println("\n\nHere comes a String containing the superfluous concept names:");
        for (String name : listOfConceptnameSets.get(1)) {
            String shortName = name.substring(3, name.length());
            if (!listOfConceptnameSets.get(0).toString().contains(shortName)) {
//                System.out.println("Concept " + shortName + " not contained in SO.owl!");
                System.out.print("owl" + shortName + ", ");
            }
        }
        System.out.println("\n\n");
        System.out.println("Size of SO.owl:              " + listOfConceptnameSets.get(0).size());
        System.out.println("Size of SO-from-obo.402.owl: " + listOfConceptnameSets.get(1).size());
        if (listOfConceptnameSets.get(0).toString().equals(listOfConceptnameSets.get(1).toString())) {
            System.out.println("Sets equal.");
        } else {
            System.out.println("Sets unequal.");
        }
    }
}
