/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.statistics;

import java.util.concurrent.TimeUnit;

/**
 *
 * @author hansen
 */
public class OverviewLogEntry {

    private String ontologyName;
    private int noOfConceptnames;
    private int noOfRolenames;
    private int noOfAxioms;
    private int noOfGammaCycles;
    private int noOfGammaCycleFOyes;
    private int noOfGammaCycleFOno;
    private int noOfGammaTimeouts;
    private int noOfOmegaTimeouts;
    private long timeNeeded;

    public OverviewLogEntry(String ontologyName) {
        this.ontologyName = ontologyName;
    }

    public int getNoOfGammaTimeouts() {
        return noOfGammaTimeouts;
    }

    public void setNoOfGammaTimeouts(int noOfGammaTimeouts) {
        this.noOfGammaTimeouts = noOfGammaTimeouts;
    }

    public int getNoOfOmegaTimeouts() {
        return noOfOmegaTimeouts;
    }

    public void setNoOfOmegaTimeouts(int noOfOmegaTimeouts) {
        this.noOfOmegaTimeouts = noOfOmegaTimeouts;
    }

    public int getNoOfConceptnames() {
        return noOfConceptnames;
    }

    public void setNoOfConceptnames(int noOfConceptnames) {
        this.noOfConceptnames = noOfConceptnames;
    }

    public int getNoOfRolenames() {
        return noOfRolenames;
    }

    public void setNoOfRolenames(int noOfRolenames) {
        this.noOfRolenames = noOfRolenames;
    }

    public int getNoOfAxioms() {
        return noOfAxioms;
    }

    public void setNoOfAxioms(int noOfAxioms) {
        this.noOfAxioms = noOfAxioms;
    }

    public int getNoOfGammaCycles() {
        return noOfGammaCycles;
    }

    public void setNoOfGammaCycles(int noOfGammaCycles) {
        this.noOfGammaCycles = noOfGammaCycles;
    }

    public int getNoOfGammaCycleFOyes() {
        return noOfGammaCycleFOyes;
    }

    public void setNoOfGammaCycleFOyes(int noOfGammaCycleFOyes) {
        this.noOfGammaCycleFOyes = noOfGammaCycleFOyes;
    }

    public int getNoOfGammaCycleFOno() {
        return noOfGammaCycleFOno;
    }

    public void setNoOfGammaCycleFOno(int noOfGammaCycleFOno) {
        this.noOfGammaCycleFOno = noOfGammaCycleFOno;
    }

    public long getTimeNeeded() {
        return timeNeeded;
    }

    public void setTimeNeeded(long timeNeeded) {
        this.timeNeeded = timeNeeded;
    }

//    private int noOfConceptnames;
//    private int noOfAxioms;
//    private int noOfGammaCycles;
//    private int noOfGammaCycleFOyes;
//    private int noOfGammaCycleFOno;
    @Override
    public String toString() {

        String result = "\n" + ontologyName + ":\n"
                + "  # of axioms:         " + noOfAxioms + "\n"
                + "  # of CNs:            " + noOfConceptnames + "\n"
                + "  # of RNs:            " + noOfRolenames + "\n"
                + "  # of FOrewr CNs:     " + (noOfConceptnames - noOfGammaTimeouts - noOfOmegaTimeouts - noOfGammaCycleFOno) + "\n"
                + "  # of Gamma timeouts: " + noOfGammaTimeouts + "\n"
                + "  # of Omega timeouts: " + noOfOmegaTimeouts + "\n"
                + "  # of Gamma cycles:   " + noOfGammaCycles + "\n"
                + "    # of FOno:           " + noOfGammaCycleFOno + "\n"
                + "    # of FOyes:          " + noOfGammaCycleFOyes + "\n"
                + "\n"
                + "Total time: " + String.format("%d min, %d sec",
                        TimeUnit.MILLISECONDS.toMinutes(timeNeeded),
                        TimeUnit.MILLISECONDS.toSeconds(timeNeeded)
                        - TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(timeNeeded))) 
                + " ("+ timeNeeded + " ms)";

        return result;
    }
}
