/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.statistics;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;

/**
 *
 * @author seylan
 */
public class OverviewLogger {

    private static OverviewLogger instance;
    private PrintWriter writer;
    private String overviewFilename;
    private String detailsFilename;
    private final String logDir = "./grind-logs/";

    private OverviewLogger() {
    }

    public static void initialize(String s, String date, boolean logging) {
        File ontFile = new File(s);
        instance = new OverviewLogger();

        if (logging) {
            File logdirFile = new File(instance.logDir);
            logdirFile.mkdir();
            instance.overviewFilename = instance.logDir + date + "-"
                    + ontFile.getName().replace(".owl", "").replace(".obo", "")
                    + "-overview.txt";
            instance.detailsFilename = instance.logDir + date + "-"
                    + ontFile.getName().replace(".owl", "").replace(".obo", "")
                    + "-details.txt";
            try {
                instance.writer = new PrintWriter(instance.overviewFilename);
                DetailsLogger.getInstance().setWriter(new PrintWriter(instance.detailsFilename));
            } catch (FileNotFoundException e) {
                System.err.println(e);
                System.exit(-1);
            }
        }
    }

    public static OverviewLogger getInstance() {
        if (instance == null) {
            System.err.println("In OverviewLogger: instance not initialized. Aborting.");
            System.exit(-1);
        }
        return instance;
    }

    public void writeLogEntry(OverviewLogEntry entry) {
        writer.println(entry);
        writer.flush();
    }

    public void close() {
        if (writer != null) {
            writer.close();
        }
    }

}
