/*
 * Copyright 2014, 2015 Peter Hansen and İnanç Seylan.
 * 
 * This file is part of Grind.
 * 
 * Grind is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Grind is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Grind.  If not, see <http://www.gnu.org/licenses/>.
 */
package de.unibremen.informatik.tdki.grind.utilities.syntax;

import java.io.File;
import org.semanticweb.owlapi.apibinding.OWLManager;
import org.semanticweb.owlapi.model.AxiomType;
import org.semanticweb.owlapi.model.OWLClassExpression;
import org.semanticweb.owlapi.model.OWLEquivalentClassesAxiom;
import org.semanticweb.owlapi.model.OWLOntology;
import org.semanticweb.owlapi.model.OWLOntologyAlreadyExistsException;
import org.semanticweb.owlapi.model.OWLOntologyCreationException;
import org.semanticweb.owlapi.model.OWLOntologyManager;
import org.semanticweb.owlapi.model.OWLSubClassOfAxiom;
import org.semanticweb.owlapi.model.UnloadableImportException;

/**
 *
 * @author seylan
 */
public class OntologyELSyntaxCheck {

    public static void main(String[] args) throws OWLOntologyCreationException {
        OWLOntologyManager manager = OWLManager.createOWLOntologyManager();
        File folder = new File("ontologies/elk");
        File[] listOfFiles = folder.listFiles();
        ELConceptVisitor visitor = new ELConceptVisitor();
        for (File ontSource : listOfFiles) {
            if (!ontSource.isFile()) {
                continue;
            }
            OWLOntology ontology = null;
            boolean ontologyFine = false;
            try {
                ontology = manager.loadOntologyFromOntologyDocument(ontSource);
                for (OWLEquivalentClassesAxiom axiom : ontology.getAxioms(AxiomType.EQUIVALENT_CLASSES)) {
                    for (OWLClassExpression c : axiom.getClassExpressions()) {
                        c.accept(visitor);
                    }
                }
                for (OWLSubClassOfAxiom axiom : ontology.getAxioms(AxiomType.SUBCLASS_OF)) {
                    axiom.getSubClass().accept(visitor);
                    axiom.getSuperClass().accept(visitor);
                }
                System.out.println(ontSource.getName() + " is OK!");
                ontologyFine = true;
            } catch (UnsupportedOperationException e) {
                System.err.println(ontSource.getName() + " is BAD!");
                System.err.println(e);
                ontologyFine = false;
            } catch (UnloadableImportException e) {
                System.err.println(ontSource.getName() + " has unloadable import.");
                ontologyFine = false;
            } catch (OWLOntologyAlreadyExistsException e) {
                System.err.println(ontSource.getName() + " already exists.");
                ontologyFine = false;
            } finally {
                if (ontology != null) {
                    manager.removeOntology(ontology);
                }
                if (!ontologyFine) {
                    ontSource.renameTo(new File("ontologies/elk-problematic/" + ontSource.getName()));
                }
            }
        }
    }
}
